@@ -2,6 +2,7 @@ use num::bigint::BigInt;
 use num::traits::cast::ToPrimitive;
 use std::borrow::Cow;
 use std::cmp::Ordering;
+use std::convert::From;
 use std::convert::TryFrom;
 use std::convert::TryInto;
 use std::fmt::Debug;
@@ -217,9 +218,11 @@ impl<N: NestedValue<D>, D: Domain> From<i64> for Value<N, D> { fn from(v: i64) -
 impl<N: NestedValue<D>, D: Domain> From<u128> for Value<N, D> { fn from(v: u128) -> Self { Value::SignedInteger(SignedInteger::from(v)) } }
 impl<N: NestedValue<D>, D: Domain> From<i128> for Value<N, D> { fn from(v: i128) -> Self { Value::SignedInteger(SignedInteger::from(v)) } }
 impl<N: NestedValue<D>, D: Domain> From<&BigInt> for Value<N, D> { fn from(v: &BigInt) -> Self { Value::SignedInteger(SignedInteger::from(Cow::Borrowed(v))) } }
+impl<N: NestedValue<D>, D: Domain> From<&SignedInteger> for Value<N, D> { fn from(v: &SignedInteger) -> Self { Value::SignedInteger(v.clone()) } }
 
 impl<N: NestedValue<D>, D: Domain> From<&str> for Value<N, D> { fn from(v: &str) -> Self { Value::String(String::from(v)) } }
 impl<N: NestedValue<D>, D: Domain> From<String> for Value<N, D> { fn from(v: String) -> Self { Value::String(v) } }
+impl<N: NestedValue<D>, D: Domain> From<&String> for Value<N, D> { fn from(v: &String) -> Self { Value::String(v.to_owned()) } }
 
 impl<N: NestedValue<D>, D: Domain> From<&[u8]> for Value<N, D> { fn from(v: &[u8]) -> Self { Value::ByteString(Vec::from(v)) } }
 // impl<N: NestedValue<D>, D: Domain> From<Vec<u8>> for Value<N, D> { fn from(v: Vec<u8>) -> Self { Value::ByteString(v) } }
@@ -720,6 +723,22 @@ impl<N: NestedValue<D>, D: Domain> Value<N, D> {
         self.as_dictionary().ok_or_else(|| self.expected(ExpectedKind::Dictionary))
     }
 
+    pub fn is_embedded(&self) -> bool {
+        self.as_embedded().is_some()
+    }
+
+    pub fn as_embedded(&self) -> Option<&D> {
+        if let Value::Domain(ref d) = *self {
+            Some(d)
+        } else {
+            None
+        }
+    }
+
+    pub fn to_embedded(&self) -> Result<&D, Error> {
+        self.as_embedded().ok_or_else(|| self.expected(ExpectedKind::Embedded))
+    }
+
     pub fn copy_via<M: NestedValue<E>, E: Domain, F>(&self, f: &F) -> Value<M, E>
     where
         F: Fn(&D) -> Value<M, E>
